library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity top is

Port (IO						: in std_logic_vector(51 downto 0);
		led					: out std_logic_vector(3 downto 0);
		btn					: in std_logic_vector(1 downto 0);
		spkr					: out std_logic;
		rx						: out std_logic;
		tx						: in std_logic;
		clk					: in std_logic);
end top;

architecture Behavioral of top is

component uart_rx
port (
    i_clk       : in  std_logic;
    i_rx_serial : in  std_logic;
    o_rx_dv     : out std_logic;
    o_rx_byte   : out std_logic_vector(7 downto 0)
    );
END component;

component uart_tx
  port (
    i_clk       : in  std_logic;
    i_tx_dv     : in  std_logic;
    i_tx_byte   : in  std_logic_vector(7 downto 0);
    o_tx_active : out std_logic;
    o_tx_serial : out std_logic;
    o_tx_done   : out std_logic
    );
END component;

signal count		: std_logic_vector(22 downto 0);
signal ios			: std_logic_vector(51 downto 0);
signal count2		: std_logic_vector(1 downto 0);
signal btn_st		: std_logic;
signal data			: std_logic_vector(7 downto 0);
signal dv			: std_logic;
signal sound_1		: std_logic;
signal sound_2		: std_logic;
signal sound_3		: std_logic;

begin

	uarttx: uart_tx
	port map(	i_clk			=> clk,
					i_tx_dv		=> dv,
					i_tx_byte	=> data,
--					o_tx_active => ,
--					o_tx_done 	=> ,
					o_tx_serial	=> rx);

	uartrx: uart_rx
	port map(	i_clk			=> clk,
					i_rx_serial => tx,
					o_rx_dv 		=> dv,
					o_rx_byte	=> data);

btns: process (btn)
	begin
	if btn(0) = '0' then btn_st <= '1';
	end if;
	if btn(1) = '0' then btn_st <= '0';
	end if;
	end process;

count_proc: process(clk)
	begin
		if clk'event and clk = '1' then
			count <= count + '1';
		end if;
	end process;

count2_proc: process(count(22))
	begin
		if count(22)'event and count(22) = '1' then
		if (btn_st='0')		then count2 <= count2 + '1';
		elsif (btn_st='1')	then count2 <= count2 - '1';
		end if;
		end if;
	end process;

leds: process(count2)
	begin
   case count2 is 
      when "00" => led <= "0001";
		when "01" => led <= "0010";
		when "10" => led <= "0100";
		when "11" => led <= "1000";
      when others => led <= "0000";
   end case;
	end process;

sound1_btn: process(btn_st,count,count2,data)
	begin
	if data="00000000" then
	if	btn_st='0' then sound_1 <= count(20) and count(11) and count2(0) and count2(1);
	elsif btn_st='1' then sound_1 <= count(20) and count(13) and count2(0) and count2(1);		
	end if;
	end if;
	end process;

sound2_uart: process (clk, data, count, count2)
begin
if (count2(0) = '0') then sound_2<='0';
elsif (clk'event and clk='1') then
	case data is
	when "00110000" =>sound_2<=count(20); --0
	when "00110001" =>sound_2<=count(19); --1
	when "00110010" =>sound_2<=count(18); --2
	when "00110011" =>sound_2<=count(17); --3
	when "00110100" =>sound_2<=count(16); --4
	when "00110101" =>sound_2<=count(15); --5
	when "00110110" =>sound_2<=count(14); --6
	when "00110111" =>sound_2<=count(13); --7
	when "00111000" =>sound_2<=count(12); --8
	when "00111001" =>sound_2<=count(11); --9
	when others =>sound_2<='0';
	end case;
	end if;
	end process;

sound3_IO: process (ios,count)
begin
	case ios is
	when X"0000000000001" =>sound_3<=count(16);
	when X"0000000000002" =>sound_3<=count(16);
	when X"0000000000004" =>sound_3<=count(16);
	when X"0000000000008" =>sound_3<=count(16);
	when X"0000000000010" =>sound_3<=count(16);
	when X"0000000000020" =>sound_3<=count(16);
	when X"0000000000040" =>sound_3<=count(16);
	when X"0000000000080" =>sound_3<=count(16);
	when X"0000000000100" =>sound_3<=count(16);
	when X"0000000000200" =>sound_3<=count(16);
	when X"0000000000400" =>sound_3<=count(16);
	when X"0000000000800" =>sound_3<=count(16);
	when X"0000000001000" =>sound_3<=count(16);
	when X"0000000002000" =>sound_3<=count(16);
	when X"0000000004000" =>sound_3<=count(16);
	when X"0000000008000" =>sound_3<=count(16);
	when X"0000000010000" =>sound_3<=count(16);
	when X"0000000020000" =>sound_3<=count(16);
	when X"0000000040000" =>sound_3<=count(16);
	when X"0000000080000" =>sound_3<=count(16);
	when X"0000000100000" =>sound_3<=count(16);
	when X"0000000200000" =>sound_3<=count(16);
	when X"0000000400000" =>sound_3<=count(16);
	when X"0000000800000" =>sound_3<=count(16);
	when X"0000001000000" =>sound_3<=count(16);
	when X"0000002000000" =>sound_3<=count(16);
	when X"0000004000000" =>sound_3<=count(16);
	when X"0000008000000" =>sound_3<=count(16);
	when X"0000010000000" =>sound_3<=count(16);
	when X"0000020000000" =>sound_3<=count(16);
	when X"0000040000000" =>sound_3<=count(16);
	when X"0000080000000" =>sound_3<=count(16);
	when X"0000100000000" =>sound_3<=count(16);
	when X"0000200000000" =>sound_3<=count(16);
	when X"0000400000000" =>sound_3<=count(16);
	when X"0000800000000" =>sound_3<=count(16);
	when X"0001000000000" =>sound_3<=count(16);
	when X"0002000000000" =>sound_3<=count(16);
	when X"0004000000000" =>sound_3<=count(16);
	when X"0008000000000" =>sound_3<=count(16);
	when X"0010000000000" =>sound_3<=count(16);
	when X"0020000000000" =>sound_3<=count(16);
	when X"0040000000000" =>sound_3<=count(16);
	when X"0080000000000" =>sound_3<=count(16);
	when X"0100000000000" =>sound_3<=count(16);
	when X"0200000000000" =>sound_3<=count(16);
	when X"0400000000000" =>sound_3<=count(16);
	when X"0800000000000" =>sound_3<=count(16);
	when X"1000000000000" =>sound_3<=count(16);
	when X"2000000000000" =>sound_3<=count(16);
	when X"4000000000000" =>sound_3<=count(16);
	when X"8000000000000" =>sound_3<=count(16);
	when others =>sound_3<='0';
	end case;
	end process;

ios(50 downto 0) <= io(50 downto 0);
ios(51) <= not io(51);
spkr <= sound_1 or sound_2 or sound_3;

end Behavioral;